package database

import (
	"database/sql"
	"fmt"

	"git.zom.bi/fanir/feedizer/cmd/feedizer/config"
	"git.zom.bi/fanir/feedizer/models/migrations"
	"github.com/golang-migrate/migrate/v4"
	"github.com/golang-migrate/migrate/v4/database/postgres"
	"github.com/golang-migrate/migrate/v4/source/iofs"
)

var LogName string

func Open(c config.DatabaseConfig) (*sql.DB, error) {
	var dsn string
	if c.Socket != "" {
		dsn = fmt.Sprintf("postgres://%s:%s@%s/%s?%s", c.Username, c.Password, c.Socket, c.Database, c.Options)
		LogName = fmt.Sprintf("%s/%s", c.Socket, c.Database)
	} else {
		dsn = fmt.Sprintf("postgres://%s:%s@%s:%d/%s?%s", c.Username, c.Password, c.Host, c.Port, c.Database, c.Options)
		LogName = fmt.Sprintf("%s:%d/%s", c.Host, c.Port, c.Database)
	}
	return sql.Open("postgres", dsn)
}

func Migrate(db *sql.DB) error {
	source, err := iofs.New(migrations.FS, ".")
	if err != nil {
		return err
	}

	driver, err := postgres.WithInstance(db, &postgres.Config{})
	if err != nil {
		return err
	}

	m, err := migrate.NewWithInstance("iofs", source, "postgres", driver)
	if err != nil {
		return err
	}

	return m.Up()
}

// Purgeable allows purging the entire database by undoing all migrations.
// This is deliberately non-straightforward to use to prevent accidental purging.
// Call .Purge() on the instance to do the thing.
type Purgeable struct{ *sql.DB }

func (p Purgeable) Purge() error {
	source, err := iofs.New(migrations.FS, ".")
	if err != nil {
		return err
	}

	driver, err := postgres.WithInstance(p.DB, &postgres.Config{})
	if err != nil {
		return err
	}

	m, err := migrate.NewWithInstance("iofs", source, "postgres", driver)
	if err != nil {
		return err
	}

	return m.Down()
}
